@@ -1,26 +1,31 @@
 from .cached_property import cached_property
 
-QUERY_TYPE = b'qt'
-COLUMN = b'c'
-FROM = b'f'
-WHERE = b'w'
-HAVING = b'h'
-GROUP_BY = b'gb'
-ORDER_BY = b'ob'
-TOP = b'top'
-LIMIT = b'l'
-OFFSET = b'os'
-EXTRA = b'ex'
-TABLE = b't'
-SET = b's'
-RETURNING = b'r'
-DISTINCT_ON = b'do'
-JOIN_LATERAL = b'jl'
+QUERY_TYPE = 'qt'
+COLUMN = 'c'
+FROM = 'f'
+WHERE = 'w'
+HAVING = 'h'
+GROUP_BY = 'gb'
+ORDER_BY = 'ob'
+TOP = 'top'
+LIMIT = 'l'
+OFFSET = 'os'
+EXTRA = 'ex'
+TABLE = 't'
+SET = 's'
+RETURNING = 'r'
+DISTINCT_ON = 'do'
+JOIN_LATERAL = 'jl'
 
-SELECT_QT = b's'
-UPDATE_QT = b'u'
-DELETE_QT = b'd'
-INSERT_QT = b'i'
+SELECT_QT = 's'
+UPDATE_QT = 'u'
+DELETE_QT = 'd'
+INSERT_QT = 'i'
+
+INNER_JOIN = 'INNER_JOIN'
+LEFT_JOIN = 'LEFT_JOIN'
+RIGHT_JOIN = 'RIGHT_JOIN'
+FULL_JOIN = 'FULL_JOIN'
 
 SEP = '\n       '
 WHERE_SEP = ' AND\n       '
@@ -32,6 +37,10 @@ INSERT_COLUMNS_SEP = ',\n   '
 INSERT_VALUES_SEP = ',\n          '
 
 
+class _default:
+    pass
+
+
 class NormAsIs:
     def __init__(self, value):
         self.value = value
@@ -216,9 +225,10 @@ class Query:
 
     @property
     def binds(self):
+        isasis = NormAsIs.isasis
         operative_binds = {}
         for key, value in self.bind_items:
-            if NormAsIs.isasis(value):
+            if isasis(value):
                 operative_binds.pop(key, None)
                 continue
             operative_binds[key] = value
@@ -247,6 +257,8 @@ class Query:
 
     @property
     def query(self):
+        isasis = NormAsIs.isasis
+
         if self._query is None:
             self._query = compile(self.build_chain(), self.query_type)
         query = self._query
@@ -256,7 +268,7 @@ class Query:
             final_binds[key] = value
         for key in sorted(final_binds.keys(), key=len, reverse=True):
             value = final_binds[key]
-            if NormAsIs.isasis(value):
+            if isasis(value):
                 query = query.replace(self.bnd(key), value.value)
         return query
 
@@ -302,11 +314,33 @@ class _SELECT_UPDATE(Query):
             s.chain.append((FROM, (stmt, False, None, None)))
         return s
 
-    def JOIN(self, stmt, ON=None, USING=None, outer=False):
-        if outer:
-            keyword = 'LEFT JOIN'
-        else:
+    def JOIN(self,
+             stmt,
+             ON=None,
+             USING=None,
+             outer=_default,
+             join_type=INNER_JOIN):
+
+        if outer is not _default:
+            import warnings
+            warnings.warn(
+                'JOIN will stop accepting outer as an argument soon, '
+                'use the named methods (such as .LEFTJOIN) instead')
+            if outer:
+                join_type = LEFT_JOIN
+            else:
+                join_type = INNER_JOIN
+
+        if join_type == INNER_JOIN:
             keyword = 'JOIN'
+        elif join_type == LEFT_JOIN:
+            keyword = 'LEFT JOIN'
+        elif join_type == RIGHT_JOIN:
+            keyword = 'RIGHT JOIN'
+        elif join_type == FULL_JOIN:
+            keyword = 'FULL JOIN'
+        else:
+            raise BogusQuery(f"Unknown join type {join_type!r}")
         if ON is not None and USING is not None:
             raise BogusQuery("You can't specify both ON and USING.")
         elif ON is not None:
@@ -314,7 +348,10 @@ class _SELECT_UPDATE(Query):
             criteria = ON
         elif USING is not None:
             op = 'USING'
-            criteria = USING
+            if isinstance(USING, str):
+                criteria = f'({USING})'
+            else:
+                criteria = '(' + ', '.join(USING) + ')'
         else:
             raise BogusQuery('No join criteria specified.')
 
@@ -323,7 +360,13 @@ class _SELECT_UPDATE(Query):
         return s
 
     def LEFTJOIN(self, *args, **kw):
-        return self.JOIN(*args, outer=True, **kw)
+        return self.JOIN(*args, join_type=LEFT_JOIN, **kw)
+
+    def RIGHTJOIN(self, *args, **kw):
+        return self.JOIN(*args, join_type=RIGHT_JOIN, **kw)
+
+    def FULLJOIN(self, *args, **kw):
+        return self.JOIN(*args, join_type=FULL_JOIN, **kw)
 
     def RETURNING(self, *args):
         s = self.child()
@@ -448,10 +491,6 @@ class DELETE(_SELECT_UPDATE):
             self.chain.append((TABLE, table))
 
 
-class _default:
-    pass
-
-
 class INSERT:
     bind_prefix = '%('
     bind_postfix = ')s'
@@ -482,6 +521,7 @@ class INSERT:
 
     @property
     def binds(self):
+        isasis = NormAsIs.isasis
         binds = {}
         if self.statement:
             binds.update(self.statement.binds)
@@ -498,10 +538,10 @@ class INSERT:
                 key = f'{col_name}_{index}'
 
                 if col_name in d:
-                    if not NormAsIs.isasis(d[col_name]):
+                    if not isasis(d[col_name]):
                         binds[key] = d[col_name]
                 else:
-                    if not NormAsIs.isasis(self.default):
+                    if not isasis(self.default):
                         binds[key] = self.default
 
         return binds
@@ -537,6 +577,7 @@ class INSERT:
         return f'{self.bind_prefix}{col_name}_{index}{self.bind_postfix}'
 
     def _query(self, data):
+        isasis = NormAsIs.isasis
         q = 'INSERT INTO %s ' % self.table
 
         if self.columns:
@@ -558,7 +599,7 @@ class INSERT:
                 last_col_ix = len(self.columns) - 1
                 for ix, col_name in enumerate(self.columns):
                     col_val = d.get(col_name, self.default)
-                    if NormAsIs.isasis(col_val):
+                    if isasis(col_val):
                         q += col_val.value
                     else:
                         q += self._bind_param_name(col_name, index)
