@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from itertools import groupby
 
 
@@ -8,14 +6,17 @@ class RowsProxy(object):
         self.rows = rows
         self.column_names = column_names
 
+    def __len__(self):
+        return len(self.rows)
+
     def __call__(self, *args):
         def key_func(row):
             if len(args) == 1:
                 return row.get(args[0])
-            l = []
+            cols = []
             for key in args:
-                l.append(row.get(key))
-            return tuple(l)
+                cols.append(row.get(key))
+            return tuple(cols)
 
         for key, sub_rows_iter in groupby(self, key=key_func):
             yield key, RowsProxy(sub_rows_iter)
@@ -27,3 +28,6 @@ class RowsProxy(object):
                 yield row
             else:
                 yield dict(zip(column_names, row))
+
+
+__all__ = [RowsProxy]
